#include <iostream>
#include "param_loader.h"

parameter_loader_base::parameter_loader_base()
{
    lua.open_libraries(sol::lib::base, sol::lib::math);
    init();
}

void
parameter_loader_base::init(void)
{
    lua["const"] = lua.create_table();

    lua["sim"] = lua.create_table();
    lua["sim"]["approx_order"] = 1;
    lua["sim"]["geom_order"] = 1;
    lua["sim"]["use_gpu"] = 0;
    //lua["sim"]["name"];
    //lua["sim"]["dt"];
    //lua["sim"]["timesteps"];
    //lua["sim"]["gmsh_model"];

    lua["materials"] = lua.create_table();
    lua["bndconds"] = lua.create_table();
    lua["ifaceconds"] = lua.create_table();

    lua["postpro"] = lua.create_table();
    lua["postpro"]["silo_output_rate"] = 0;
    lua["postpro"]["cycle_print_rate"] = 10;
}

bool
parameter_loader_base::validate_simulation_params(void) const
{
    bool success = true;

    auto sim_name = lua["sim"]["name"];
    if (not sim_name.valid())
    {
        std::cout << "[CONFIG] 'sim.name' not set" << std::endl;
        success = false;
    } 

    auto sim_dt = lua["sim"]["dt"];
    if (not sim_dt.valid())
    {
        std::cout << "[CONFIG] 'sim.dt' not set" << std::endl;
        success = false;
    } 

    auto sim_timesteps = lua["sim"]["timesteps"];
    if (not sim_timesteps.valid())
    {
        std::cout << "[CONFIG] 'sim.timesteps' not set" << std::endl;
        success = false;
    } 

    auto sim_gmshmodel = lua["sim"]["gmsh_model"];
    if (not sim_gmshmodel.valid())
    {
        std::cout << "[CONFIG] 'sim.gmsh_model' not set" << std::endl;
        success = false;
    } 

    return success;
}

bool
parameter_loader_base::load_file(const std::string& fn)
{
    auto script = lua.script_file(fn);
    if (not script.valid())
        return false;

    return validate_simulation_params();
}

int
parameter_loader_base::sim_approxorder(void) const
{
    return lua["sim"]["approx_order"];
}

int
parameter_loader_base::sim_geomorder(void) const
{
    return lua["sim"]["geom_order"];
}

double
parameter_loader_base::sim_dt(void) const
{
    return lua["sim"]["dt"];
}

size_t
parameter_loader_base::sim_timesteps(void) const
{
    return lua["sim"]["timesteps"];
}

std::string
parameter_loader_base::sim_name(void) const
{
    return lua["sim"]["name"];
}

std::string
parameter_loader_base::sim_gmshmodel(void) const
{
    return lua["sim"]["gmsh_model"];
}

bool
parameter_loader_base::sim_usegpu(void) const
{
    int ug = lua["sim"]["use_gpu"];
    return ug != 0;
}

size_t
parameter_loader_base::postpro_cyclePrintRate(void) const
{
    return lua["postpro"]["cycle_print_rate"];
}

size_t
parameter_loader_base::postpro_siloOutputRate(void) const
{
    return lua["postpro"]["silo_output_rate"];
}

